Package: sjmisc
Type: Package
Encoding: UTF-8
Title: Data and Variable Transformation Functions
Version: 2.8.11
Authors@R: c(person("Daniel", "Lüdecke", role = c("aut", "cre"), email = "d.luedecke@uke.de", comment = c(ORCID = "0000-0002-8895-3206")),
    person("Iago", "Giné-Vázquez", role = c("ctb"), email = "i.gine@pssjd.org"),
    person("Alexander", "Bartel", role = "ctb", email = "alexander.bartel@fu-berlin.de", comment = c(ORCID = "0000-0002-1280-6138")))
Maintainer: Daniel Lüdecke <d.luedecke@uke.de>
Description: Collection of miscellaneous utility functions, supporting data
    transformation tasks like recoding, dichotomizing or grouping variables,
    setting and replacing missing values. The data transformation functions
    also support labelled data, and all integrate seamlessly into a
    'tidyverse'-workflow.
License: GPL-3
Depends: R (>= 3.4)
Imports: dplyr, insight, datawizard, magrittr, methods, purrr, rlang,
        sjlabelled (>= 1.1.1), stats, tidyselect, utils
Suggests: ggplot2, graphics, haven (>= 2.0.0), mice, nnet, sjPlot,
        sjstats, knitr, rmarkdown, stringdist, testthat, tidyr
URL: https://strengejacke.github.io/sjmisc/
BugReports: https://github.com/strengejacke/sjmisc/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-07-30 09:59:14 UTC; Daniel
Author: Daniel Lüdecke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8895-3206>),
  Iago Giné-Vázquez [ctb],
  Alexander Bartel [ctb] (ORCID: <https://orcid.org/0000-0002-1280-6138>)
Repository: CRAN
Date/Publication: 2025-07-30 10:50:02 UTC
Built: R 4.6.0; ; 2026-04-28 23:22:30 UTC; unix
