# Generated by roxygen2: do not edit by hand

S3method(all_na,data.frame)
S3method(all_na,default)
S3method(all_na,list)
S3method(as.data.frame,sjmisc_frq)
S3method(big_mark,data.frame)
S3method(big_mark,default)
S3method(big_mark,list)
S3method(center,default)
S3method(center,mids)
S3method(dicho,default)
S3method(dicho,mids)
S3method(format,sjmisc_frq)
S3method(is_even,data.frame)
S3method(is_even,default)
S3method(is_even,list)
S3method(is_float,data.frame)
S3method(is_float,default)
S3method(is_float,list)
S3method(is_odd,data.frame)
S3method(is_odd,default)
S3method(is_odd,list)
S3method(is_whole,data.frame)
S3method(is_whole,default)
S3method(is_whole,list)
S3method(prcn,data.frame)
S3method(prcn,default)
S3method(print,sj_has_na)
S3method(print,sj_merge.imp)
S3method(print,sjmisc_descr)
S3method(print,sjmisc_frq)
S3method(print,sjmisc_frq2)
S3method(print,sjmisc_grpdescr)
S3method(print_html,sjmisc_frq)
S3method(print_md,sjmisc_frq)
S3method(rec,default)
S3method(rec,mids)
S3method(recode_to,default)
S3method(recode_to,mids)
S3method(round_num,data.frame)
S3method(round_num,default)
S3method(round_num,list)
S3method(row_count,default)
S3method(row_count,mids)
S3method(row_means,default)
S3method(row_means,mids)
S3method(row_sums,default)
S3method(row_sums,mids)
S3method(std,default)
S3method(std,mids)
S3method(to_long,default)
S3method(to_long,mids)
S3method(total_mean,data.frame)
S3method(trim,data.frame)
S3method(trim,default)
S3method(trim,list)
export("%>%")
export("%nin%")
export(add_case)
export(add_columns)
export(add_id)
export(add_rows)
export(add_variables)
export(all_na)
export(big_mark)
export(center)
export(center_if)
export(clean_values)
export(col_count)
export(complete_cases)
export(complete_vars)
export(count_na)
export(de_mean)
export(descr)
export(dicho)
export(dicho_if)
export(empty_cols)
export(empty_rows)
export(find_in_data)
export(find_var)
export(flat_table)
export(frq)
export(group_labels)
export(group_labels_if)
export(group_str)
export(group_var)
export(group_var_if)
export(has_na)
export(incomplete_cases)
export(incomplete_vars)
export(is_cross_classified)
export(is_crossed)
export(is_empty)
export(is_even)
export(is_float)
export(is_nested)
export(is_num_chr)
export(is_num_fac)
export(is_odd)
export(is_whole)
export(merge_df)
export(merge_imputations)
export(move_columns)
export(numeric_to_factor)
export(prcn)
export(print_html)
export(print_md)
export(rec)
export(rec_if)
export(rec_pattern)
export(recode_to)
export(recode_to_if)
export(ref_lvl)
export(remove_cols)
export(remove_empty_cols)
export(remove_empty_rows)
export(remove_var)
export(rename_columns)
export(rename_variables)
export(replace_columns)
export(replace_na)
export(reshape_longer)
export(rotate_df)
export(round_num)
export(row_count)
export(row_means)
export(row_sums)
export(seq_col)
export(seq_row)
export(set_na)
export(set_na_if)
export(shorten_string)
export(split_var)
export(split_var_if)
export(spread_coef)
export(std)
export(std_if)
export(str_contains)
export(str_end)
export(str_find)
export(str_start)
export(tidy_values)
export(to_character)
export(to_dummy)
export(to_factor)
export(to_label)
export(to_long)
export(to_numeric)
export(to_value)
export(total_mean)
export(trim)
export(typical_value)
export(var_rename)
export(var_type)
export(word_wrap)
export(zap_inf)
importFrom(insight,print_html)
importFrom(insight,print_md)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(sjlabelled,set_na)
importFrom(sjlabelled,to_character)
importFrom(sjlabelled,to_factor)
importFrom(sjlabelled,to_label)
importFrom(sjlabelled,to_numeric)
