% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3-methods.R, R/re-exports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{print_md}
\alias{print_html}
\alias{to_character}
\alias{to_label}
\alias{to_numeric}
\alias{to_factor}
\alias{\%>\%}
\alias{set_na}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{insight}{\code{\link[insight:display]{print_html}}, \code{\link[insight:display]{print_md}}}

  \item{magrittr}{\code{\link[magrittr:pipe]{\%>\%}}}

  \item{sjlabelled}{\code{\link[sjlabelled]{set_na}}, \code{\link[sjlabelled:as_label]{to_character}}, \code{\link[sjlabelled:as_factor]{to_factor}}, \code{\link[sjlabelled:as_label]{to_label}}, \code{\link[sjlabelled:as_numeric]{to_numeric}}}
}}

